'--------------------------------------------------------------------------------------
'name                     : LightControlledBlinking.bas
'programmed by            : Claus Kuhnel
'date                     : 2008-12-29
'purpose                  : Light Controlled Blinking
'micro                    : LilyPad Arduino - ATmega168
'tested with              : V.1.12.0.0 (2010-08-08)
'suited for demo          : yes
'commercial addon needed  : no
'--------------------------------------------------------------------------------------
' Turns light emitting diodes (LED) on and off in blinking mode depending
' of the amount of ambient light.
' The switching point is defined by a constant as default and can be changed
' by serial input a valueue between 0 and 255.
'--------------------------------------------------------------------------------------
$regfile = "m168def.dat"
$crystal = 8000000
$baud = 19200

$hwstack = 32                             ' default use 32 for the hardware stack
$swstack = 20                               ' default use 10 for the SW stack
$framesize = 40                            ' default use 40 for the frame space

Debug On                                    ' for debug output on, otherwise off

Const Default_threshold = 120               ' threshold defines activation of blinking

Dim Threshold As Byte
Dim Value As Word : Value = 0       ' variable to store the valueue coming from sensor
Dim Cc As Byte                           ' variable to store serial input
Dim Channel As Byte : Channel = 0

Declare Sub Setup
Declare Sub Blinking

Setup

Do
   Cc = Ischarwaiting()            ' Check if data has been sent from host
   If Cc = 1 Then
      Threshold = Waitkey()    ' Read The Most Recent Byte(which Will Be From 0 To 255)
   End If
   Value = Getadc(channel)                   ' Read the value from Light Sensor
   Shift Value , Right , 2

   Debug "Threshold = " ; Threshold ; Spc(3) ; "Light value = " ; Value;

   If Value < Threshold Then
      Blinking : Debug Spc(3) ; "*"        ' Blinking is marked with * in debug output
   Else
      Wait 1 : Debug ""
   End If
Loop

End

'--------------------------------------------------------------------------------------

Sub Setup
   Debug ""                                  ' new line for output
   Debug "LilyPad Setup for Light Controlled Blinking..."
   Config Adc = Single , Prescaler = Auto , Reference = Avcc
   Value = Getadc(channel)                    ' First dummy ADC
   Config Pinb.5 = Output                     ' pin for internal LED
   Led Alias Portb.5
   Config Pinb.0 = Output                     ' pin for LED1
   Led1 Alias Portb.0
   Config Pinb.1 = Output                     ' pin for LED2
   Led2 Alias Portb.1
   Threshold = Default_threshold
   Debug "LilyPad is running..."
End Sub

Sub Blinking
   Led = 1                            ' LEDs ON
   Led1 = 1
   Led2 = 1
   Waitms 50                           ' waits a short moment
   Led = 0                              ' LEDs OFF
   Led1 = 0
   Led2 = 0
   Waitms 100                          ' waits a short moment
   Led = 1                              ' LEDs ON
   Led1 = 1
   Led2 = 1
   Waitms 50                          ' waits a short moment
   Led = 0                             ' LEDs OFF
   Led1 = 0
   Led2 = 0
   Wait 1                               ' waits for a second
End Sub