$regfile = "m8def.dat"                  ' ATmega8
$crystal = 3690000                      ' fr STK500
$baud = 19200

' HW Stack 64; Soft Stack 32; Frame Size 64   nicht optimiert

Const Device_id = &HA                   ' Device ID fr EEPROM NM24Cxx
Const Page_addr = 1                     ' verwendete Page
Const Word_addr = 5                     ' verwendete Speicherzelle
Const Ee_timeout = 50000                ' TWI TimeOut
Const Esc = 27

Dim Slave_wa As Byte                    ' Slave Write Adresse
Dim Ee_data As Byte                     ' verwendetes Datenbyte
Dim Temp As Byte

Declare Sub Twiinit(byval Clockrate As Byte)
Declare Function Twi_cmd(byval Cmd As Byte) As Byte

Declare Sub I2c_wb(byval Swa As Byte , Byval Wa As Byte , Dta As Byte)
Declare Function I2c_rb(byval Swa As Byte , Byval Wa As Byte) As Byte

Slave_wa = Device_id                    ' Berechnung der Slaveadresse
Shift Slave_wa , Left , 4

Temp = Page_addr
Shift Temp , Left

Slave_wa = Slave_wa Or Temp             ' Berechnung Slave Write Adresse

Print "EEPROM Schreiben & Lesen - Abbruch mit ESC"
Ee_data = &HA5
Call Twiinit(10)                        ' Initialisiere TWI - SCL ca. 100 kHz

Do
   Incr Ee_data                         ' Inkrementieren Datenbyte
   Print "Schreibe " ; Ee_data ; " nach Adresse " ; Word_addr
   I2c_wb Slave_wa , Word_addr , Ee_data       ' Byte in EEPROM schreiben
   If Err = 0 Then
      Print "No I2C Write Error"
   Else
      Print "I2C Write Error"
   End If

   Temp = I2c_rb(slave_wa , Word_addr)  ' Byte aus EEPROM zurcklesen
   If Err = 0 Then
      Print "No I2C Read Error"
   Else
      Print "I2C Read Error"
   End If
   Print "Lese " ; Temp ; " von Adresse " ; Word_addr       ' Ausgabe gelesenes Byte
   Wait 1
   Temp = Inkey()
Loop Until Temp = Esc

Print "Ende."

End

' Initialisierung TWI Hardware
Sub Twiinit(byval Clockrate As Byte)
   Twsr = Twsr And &HFC                 ' Prescaler = 1
   Twbr = Clockrate                     ' SCL Takt
End Sub

' Schreibt ein Kommando in Register TWCR und fragt Statusregister TWSR ab
Function Twi_cmd(byval Cmd As Byte) As Byte
  Local Time_out As Word , I As Byte

  Twcr = Cmd
  Time_out = 0
  Do
     Incr Time_out
  Loop Until Twcr.twint = 1 Or Time_out = Ee_timeout

  I = Twsr
  I = I And &HF8
  Twi_cmd = I
End Function

' Schreibe Datenbyte zum I2C-Bus Device
Sub I2c_wb(byval Swa As Byte , Byval Wa As Byte , Dta As Byte)
   Local Time_out As Word

   If Twi_cmd(&Ha4) = &H08 Then         ' Startbedingung
      Twdr = Swa                        ' Sende SLA+W
      If Twi_cmd(&H84) = &H18 Then
         Twdr = Wa                      ' Sende Wordadresse
         If Twi_cmd(&H84) = &H28 Then
            Twdr = Dta                  ' Send Datenbyte
            If Twi_cmd(&H84) = &H28 Then
               Twcr = &H94              ' Stoppbedingung
               Do                       ' warte auf Ende
                  Incr Time_out         ' oder TimeOut
               Loop Until Twcr.twsto = 0 Or Time_out = Ee_timeout
            Else
               Err = 1
            End If
         Else
            Err = 1
         End If
      Else
         Err = 1
      End If
   Else
      Err = 1
   End If
End Sub

' Lese Datenbyte vom I2C-Bus Device
Function I2c_rb(byval Swa As Byte , Byval Wa As Byte) As Byte
   Local T As Byte
   Local Sra As Byte
   Local Time_out As Word

   Sra = Swa Or 1

   If Twi_cmd(&Ha4) = &H08 Then         ' Startbedingung
      Twdr = Swa                        ' Sende SLA+W
      If Twi_cmd(&H84) = &H18 Then
         Twdr = Wa                      ' Sende Wordadresse
         If Twi_cmd(&H84) = &H28 Then
            If Twi_cmd(&Ha4) = &H10 Then       ' Startbedingun
               Twdr = Swa Or 1          ' Send SLA+R
               If Twi_cmd(&H84) = &H40 Then
                  If Twi_cmd(&H84) = &H58 Then       ' Lese Datenbyte
                     T = Twdr
                     Twcr = &H94        ' Stoppbedingung
                     Do                 ' Warte auf Ende
                        Incr Time_out   ' oder TimeOut
                     Loop Until Twcr.twsto = 0 Or Time_out = Ee_timeout
                  Else
                     Err = 1
                  End If
               Else
                  Err = 1
               End If
            Else
               Err = 1
            End If
         Else
            Err = 1
         End If
      Else
         Err = 1
      End If
   Else
      Err = 1
   End If
   I2c_rb = T
End Function
